@@ -1,5 +1,12 @@
 import itertools
 import numpy
+import sys
+if sys.version_info.major >= 3:
+    _zip = zip
+    _zip_longest = itertools.zip_longest
+else:
+    _zip = itertools.izip
+    _zip_longest = itertools.izip_longest
 
 try:
     from ._core import Error, InvalidParamError
@@ -11,7 +18,7 @@ import dagpype_c
 __all__ = []
 
 
-class UnknownNamedCSVColError(Error):
+class UnknownNamedCSVColError(Error, ValueError):
     """    Indicates a name was not found in a CSV header.
     """
     def __init__(self, name):
@@ -21,7 +28,7 @@ class UnknownNamedCSVColError(Error):
         Arguments:
         name -- Name of column that could not be found.
         """
-        Error.__init__(self, 'Could not find ' + name)
+        Error.__init__(self, b'Could not find ' + name)
         self._name = name
 
     def name(self):
@@ -33,13 +40,21 @@ __all__ += ['UnknownNamedCSVColError']
 
 
 def cols_are_type(cols, type_):
+    if isinstance(cols, bytes):
+        return bytes == type_
+
     try:
         if cols is None:
             return False
         if isinstance(cols, type_):
             return True
+        if isinstance(cols[0], bytes):
+            for c in cols[1: ]:
+                if not isinstance(c, bytes):
+                    raise InvalidParamError('cols', cols, 'Must be homogeneous')
+            return type_ == bytes                    
         if isinstance(cols[0], type_):
-            for c in cols:
+            for c in cols[1: ]:
                 if not isinstance(c, type_):
                     raise InvalidParamError('cols', cols, 'Must be homogeneous')
             return True
@@ -65,13 +80,13 @@ _all_cols = 2
 
 
 def _inds_from_cols(stream, cols, types_, delimit, comment, skip_init_space):
-    if cols_are_type(cols, str):
-        single = isinstance(cols, str)
+    if cols_are_type(cols, bytes):
+        single = isinstance(cols, bytes)
         firsts = dagpype_c.line_to_tuple(
             stream, 
             delimit, comment, 1 if skip_init_space else 0)
         firsts = dict((firsts, i) for i, firsts in enumerate(firsts))
-        if isinstance(cols, str):
+        if isinstance(cols, bytes):
             cols = (cols,)
         inds = []
         for col in cols:
@@ -96,7 +111,7 @@ def _inds_from_cols(stream, cols, types_, delimit, comment, skip_init_space):
         inds = ()
         uniques, copies, max_ind = _process_inds(inds) 
     else:
-        raise InvalidParamError('cols', cols, 'Unknown')
+        raise InvalidParamError(b'cols', cols, b'Unknown')
 
     return cols, single, inds, uniques, copies, max_ind
 
@@ -117,7 +132,7 @@ def _csv_attribs(stream, cols, types_, delimit, comment, skip_init_space):
                 c_types = c_types + (_int,)
             elif t == float:
                 c_types = c_types + (_float,)
-            elif t == str:
+            elif t == bytes:
                 c_types = c_types + (_str,)
             else:
                 c_types = c_types + (_str,)
@@ -137,17 +152,18 @@ def _make_bufs(c_types, max_field_len, max_elems):
         elif t == _float:
             bufs.append( numpy.array(range(max_elems), dtype = float) )
         elif t == _str:
-            bufs.append( numpy.array(range(max_field_len * max_elems), dtype = str) )
+            bufs.append( numpy.array(range(max_field_len * max_elems), dtype = bytes) )
     return bufs
 
 
 def _copy_array(buf, type_, len_, max_field_len, max_elems):
-    if type_ == str:
+    if type_ == bytes:
         strs = []
-        for i in xrange(len_):
-            len_ = int(buf.data[max_field_len * i : max_field_len * i + 3])
-            strs.append(buf.data[max_field_len * i + 3 : max_field_len * i + 3 + len_])
-        return numpy.array(strs, dtype = str)
+        for i in range(len_):
+            len_ = 100 * int(buf[max_field_len * i]) + 10 * int(buf[max_field_len * i + 1]) + int(buf[max_field_len * i + 2])
+            str_ = type_().join(type_(buf[j]) for j in range(max_field_len * i + 3, max_field_len * i + 3 + len_))
+            strs.append(str_)
+        return numpy.array(strs, dtype = type_)
 
     if len_ < max_elems:
         return numpy.array(buf[: len_], copy = True, dtype = type_)
@@ -159,8 +175,11 @@ def array_read(stream, cols, types_, missing_vals, delimit, comment, skip_init_s
         _csv_attribs(stream, cols, types_, delimit, comment, skip_init_space)
 
     assert max_elems > 0
+    
+    def _encode(e):
+        return e if isinstance(e, bytes) else str(e).encode()                    
 
-    c_missing_vals = (str(missing_vals),) if single else tuple(str(m) for m in missing_vals)
+    c_missing_vals = (_encode(missing_vals), ) if single else tuple(_encode(m) for m in missing_vals)
     assert len(cols) == len(c_types) == len(c_missing_vals)
 
     max_field_len = dagpype_c.parser_max_field_len()
@@ -172,7 +191,7 @@ def array_read(stream, cols, types_, missing_vals, delimit, comment, skip_init_s
 
     r = dagpype_c.array_col_reader(
         stream, 
-        str.encode(delimit), str.encode(comment), 1 if skip_init_space else 0, 
+        delimit, comment, 1 if skip_init_space else 0, 
         1 if single else 0,
         inds, uniques, copies, max_ind,
         c_types, 
@@ -187,7 +206,7 @@ def array_read(stream, cols, types_, missing_vals, delimit, comment, skip_init_s
             payload = _copy_array(bufs[0], types_, len_, max_field_len, max_elems)
         else:
             payload = \
-                tuple(_copy_array(buf, type_, len_, max_field_len, max_elems) for buf, type_ in itertools.izip(bufs, types_))
+                tuple(_copy_array(buf, type_, len_, max_field_len, max_elems) for buf, type_ in _zip(bufs, types_))
         yield payload
         if len_ < max_elems:
             break
@@ -203,7 +222,7 @@ def read(stream, cols, types_, delimit, comment, skip_init_space):
 
     r = dagpype_c.col_reader(
         stream, 
-        str.encode(delimit), str.encode(comment), 1 if skip_init_space else 0, 
+        delimit, comment, 1 if skip_init_space else 0, 
         1 if single else 0,
         inds, uniques, copies, max_ind,
         c_types)
@@ -219,5 +238,5 @@ def read(stream, cols, types_, delimit, comment, skip_init_space):
         return
 
     for t in r:
-        yield tuple(_cast(a, b) for a, b in itertools.izip_longest(t, types_))
+        yield tuple(_cast(a, b) for a, b in _zip_longest(t, types_))
 
