@@ -816,13 +816,13 @@ def csv_split(
     types__ = types_
     @filters
     def _dagpype_internal_fn_act(target):
-        try:            
+        try:     
             e = (0,) if cols is None or _csv_utils.cols_are_type(cols, int) else (yield) 
             (cols_, single, inds, uniques, copies, max_ind, types_, c_types, cast_back) = \
                 _csv_utils._csv_attribs((e, ), cols, types__, delimit, comment, skip_init_space)
             r = dagpype_c.col_reader(
                 [], 
-                str.encode(delimit), str.encode(comment), 1 if skip_init_space else 0, 
+                delimit, comment, 1 if skip_init_space else 0, 
                 1 if single else 0,
                 inds, uniques, copies, max_ind,
                 c_types)
