@@ -10,7 +10,7 @@ import dagpype_c
 
 
 __all__ = []
-
+    
 
 __all__ += ['to_stream']
 def to_stream(stream, names = None, delimit = b',', line_terminator = b'\n', buf_size = 256):    
@@ -50,48 +50,73 @@ def to_stream(stream, names = None, delimit = b',', line_terminator = b'\n', buf
     
     @sinks
     def _dagpype_internal_fn_act(target):
+        class _LineWriter(object):
+            def __init__(self, stream_, line_terminator, w):
+                self._stream_ = stream_
+                self._line_terminator = line_terminator
+                self._w = w
+                self._n = 0
+                self._first = True
+                
+            def write(self, stored):
+                if self._w is not None:
+                    dagpype_c.line_writer_write(self._w, stored)
+                    return
+                self._n += len(stored)
+                self._stream_.write(
+                    line_terminator + line_terminator.join(stored) if not self._first else line_terminator.join(stored))
+                self._first = False
+            
+            def close(self):
+                if self._w is not None:
+                    return dagpype_c.line_writer_close(self._w)
+                return self._n
+                
+        def _encode(e):
+            return e if isinstance(e, bytes) else str(e).encode()                    
+                    
         def _stringify_tup(tup):
-            return delimit.join(str(e) for e in tup)
-    
+            return delimit.join(_encode(e) for e in tup)
+            
         stream_ = open(stream, 'wb') if isinstance(stream, str) else stream
         
         if names is not None:        
             stream_.write((_stringify_tup(names) if isinstance(names, tuple) else names) + line_terminator)
             stream_.flush()
-
-        writer = dagpype_c.line_writer(stream_.fileno(), line_terminator)        
+            
+        writer = _LineWriter(stream_, line_terminator, dagpype_c.line_writer(stream_.fileno(), line_terminator))     
         
         stored = []
         try:
             e = (yield)
             if isinstance(e, tuple):                
-                while True:
+                while True:      
                     stored.append(_stringify_tup(e))
                     if len(stored) > buf_size:
-                        dagpype_c.line_writer_write(writer, stored)
+                        writer.write(stored)
                         stored = []
                     e = (yield)
-            elif isinstance(e, str):
+            elif isinstance(e, bytes):
                 while True:
                     stored.append(e)
                     if len(stored) > buf_size:
-                        dagpype_c.line_writer_write(writer, stored)
+                        writer.write(stored)
                         stored = []
                     e = (yield)
             else:
                 while True:
-                    stored.append(str(e))
+                    stored.append(_encode(e))
                     if len(stored) > buf_size:
-                        dagpype_c.line_writer_write(writer, stored)
+                        writer.write(stored)
                         stored = []
                     e = (yield)
         except GeneratorExit:
             if len(stored) > 0:
-                dagpype_c.line_writer_write(writer, stored)
-            target.send( dagpype_c.line_writer_close(writer) )    
+                writer.write(stored)
+            target.send( writer.close() )    
             target.close()
 
-        if isinstance(stream, str):
+        if isinstance(stream, bytes):
             stream_.close()
 
     return _dagpype_internal_fn_act
