@@ -227,9 +227,13 @@ col_reader_parse_string(PyObject *module, PyObject *args, PyObject *keyword_args
         return NULL;
     }
    
-    if(self->copy_cols != NULL)
-        return col_reader_copy_parsed_from_inds(self, num_parsed, parsed);
-    return col_reader_copy_parsed(self, num_parsed, parsed);
+    PyObject * const tup = self->copy_cols != NULL?
+        col_reader_copy_parsed_from_inds(self, num_parsed, parsed) :
+        col_reader_copy_parsed(self, num_parsed, parsed);
+   
+    Py_DECREF(lineobj);
+
+    return tup;
 }
 
 extern "C" int
