@@ -7,13 +7,10 @@
 
 #include "dbg.hpp"
 
-#if PY_MAJOR_VERSION >= 3
-#define IS_PY3K 1
-#else // #if PY_MAJOR_VERSION >= 3
-#define IS_PY3K 0
-#endif // #if PY_MAJOR_VERSION >= 3
-
 struct PyArrayObject;
 bool is_delightful_array(const PyArrayObject * a);
 
+PyObject *
+pyint_from_long(long l);
+
 #endif // #ifndef DEFS_HPP
