@@ -23,14 +23,11 @@ struct LineWriter
 {
     PyObject_HEAD
     
-    int fileno;
-    
     string * line_terminator;      
 
+    int fileno;    
 #ifdef DAGPYPE_USE_AIO
     vector<iovec> * iovecs;
-#else // #ifdef DAGPYPE_USE_AIO
-    buf_t * buf;    
 #endif // #ifdef DAGPYPE_USE_AIO
     
     bool first;
@@ -52,8 +49,6 @@ line_writer_dealloc(LineWriter * self)
     delete self->line_terminator;
 #ifdef DAGPYPE_USE_AIO
     delete self->iovecs;
-#else // #ifdef DAGPYPE_USE_AIO
-    delete self->buf;
 #endif // #ifdef DAGPYPE_USE_AIO
 
     PyObject_GC_UnTrack(self);
@@ -64,38 +59,39 @@ line_writer_dealloc(LineWriter * self)
 extern "C" PyObject *
 line_writer(PyObject *module, PyObject *args, PyObject *keyword_args)
 {
+#ifndef DAGPYPE_USE_AIO
+    Py_RETURN_NONE;
+#endif // #ifndef DAGPYPE_USE_AIO
+
     LineWriter * const self = PyObject_GC_New(LineWriter, &LineWriterType);
     if (self == NULL) {
         PyErr_NoMemory();
         return NULL;
     }
-
-    self->fileno = -1;
+    
     self->line_terminator = NULL;
+    self->fileno = -1;
 #ifdef DAGPYPE_USE_AIO
     self->iovecs = NULL;
-#else // #ifdef DAGPYPE_USE_AIO
-    self->buf = NULL;
 #endif // #ifdef DAGPYPE_USE_AIO
     self->first = true;
 
     char * line_terminator;
     if (!PyArg_ParseTuple(
             args,
-            "is",
+            "is*",
             &self->fileno,
             &line_terminator) || self->fileno <= 0) {
+        WARN("Failed to parse stuff " << self->fileno << " " << line_terminator);
         PyErr_SetString(PyExc_TypeError, "Failed to parse stuff");
         Py_DECREF(self);
         return NULL;
     }
-        
+
     try {
         self->line_terminator = new string(line_terminator);
 #ifdef DAGPYPE_USE_AIO
         self->iovecs = new vector<iovec>();
-#else // #ifdef DAGPYPE_USE_AIO
-        self->buf = new buf_t();        
 #endif // #ifdef DAGPYPE_USE_AIO
     }
     catch(...) {
@@ -151,7 +147,9 @@ make_iovecs(LineWriter * self, PyObject * tup)
     for (Py_ssize_t i = 0; i < PySequence_Length(tup); ++i) {
         PyObject * const p = PySequence_ITEM(tup, i);
         Py_ssize_t length;
-        char * const buf = pystring_as_string(p, length);
+        long len_;
+        char * const buf = pystring_as_string(p, len_);
+        length = static_cast<Py_ssize_t>(len_);
         self->iovecs->push_back(make_iovec(buf, length));
         Py_XDECREF(p);
         if (i + 1 < PySequence_Length(tup) &&  !self->line_terminator->empty())
@@ -164,12 +162,12 @@ make_iovecs(LineWriter * self, PyObject * tup)
 
 #ifdef DAGPYPE_USE_AIO
 static void 
-write_iovecs(LineWriter * self, int fileno)
+write_iovecs(LineWriter * self)
 {    
     if (self->iovecs->empty())
         return;
 
-    const int written = writev(fileno, &(*self->iovecs)[0], self->iovecs->size());
+    const int written = writev(self->fileno, &(*self->iovecs)[0], self->iovecs->size());
     if (written < 0) {
        PyErr_SetString(PyExc_IOError, "Failed to write");
        throw runtime_error("Failed to write");
@@ -177,57 +175,6 @@ write_iovecs(LineWriter * self, int fileno)
 }
 #endif // #ifdef DAGPYPE_USE_AIO
 
-#ifndef DAGPYPE_USE_AIO
-static size_t
-make_buf(LineWriter * self, PyObject * tup)
-{
-    size_t total_len = self->first? 0 : self->line_terminator->size();        
-    for (Py_ssize_t i = 0; i < PySequence_Length(tup); ++i) {
-        PyObject * const p = PySequence_ITEM(tup, i);
-        total_len += PyString_GET_SIZE(p);
-        Py_XDECREF(p);
-        ++self->n;
-    }
-    DBG_VERIFY(PySequence_Length(tup) > 0);
-    total_len += self->line_terminator->size() * (PySequence_Length(tup) - 1);
-    
-    try {
-        self->buf->reserve(total_len);    
-    }
-    catch(...) {
-        PyErr_NoMemory();
-        throw;
-    }
-    self->buf->resize(0);
-    
-    if (!self->first)    
-        copy(self->line_terminator->begin(), self->line_terminator->end(), back_inserter(*self->buf));
-    for (Py_ssize_t i = 0; i < PySequence_Length(tup); ++i) {
-        PyObject * const p = PySequence_ITEM(tup, i);
-        Py_ssize_t length;
-        char * const buf = pystring_as_string(p, length)
-        copy(buf, buf + length, back_inserter(*self->buf));
-        Py_XDECREF(p);
-        if (i + 1 < PySequence_Length(tup))
-            copy(self->line_terminator->begin(), self->line_terminator->end(), back_inserter(*self->buf));
-    }
-    
-    return total_len;
-}
-#endif // #ifndef DAGPYPE_USE_AIO
-
-#ifndef DAGPYPE_USE_AIO
-static void 
-write_buf(LineWriter * self, int fileno, size_t total_len)
-{
-    const size_t written = write(fileno, &(*self->buf)[0], total_len);
-    if (written < total_len) {
-       PyErr_SetString(PyExc_IOError, "Failed to write");
-       throw runtime_error("Failed to write");
-    }
-}
-#endif // #ifndef DAGPYPE_USE_AIO
-
 extern "C" PyObject *
 line_writer_write(PyObject * module, PyObject * args, PyObject * keyword_args)
 {        
@@ -238,6 +185,7 @@ line_writer_write(PyObject * module, PyObject * args, PyObject * keyword_args)
             "OO",
             reinterpret_cast<LineWriter * *>(&self), 
             &tup) || self == NULL || tup == NULL || !PySequence_Check(tup)) {
+        WARN("Failed to parse stuff");
         PyErr_SetString(PyExc_TypeError, "Failed to parse stuff");
         return NULL;
     }
@@ -245,10 +193,7 @@ line_writer_write(PyObject * module, PyObject * args, PyObject * keyword_args)
     try {
 #ifdef DAGPYPE_USE_AIO
         make_iovecs(self, tup);
-        write_iovecs(self, self->fileno);
-#else // #ifdef DAGPYPE_USE_AIO
-        const size_t total_len = make_buf(self, tup);
-        write_buf(self, self->fileno, total_len);
+        write_iovecs(self);
 #endif // #ifdef DAGPYPE_USE_AIO
     }
     catch(...) {
@@ -268,6 +213,7 @@ line_writer_close(PyObject *module, PyObject *args, PyObject *keyword_args)
             args,
             "O",
             reinterpret_cast<LineWriter * * >(&self)) || self == NULL) {
+        WARN("Failed to parse stuff " << self);
         PyErr_SetString(PyExc_TypeError, "Failed to parse stuff");
         return NULL;
     }
@@ -320,3 +266,4 @@ PyTypeObject LineWriterType = {
     0,                                      /*tp_getset*/
 };
 
+
