@@ -9,7 +9,7 @@ __all__ = []
 
 
 __all__ += ['chunks_to_stream']
-def chunks_to_stream(stream, names = None, fmt = '%.18e', delimit = b',', line_terminator = b'\n'):    
+def chunks_to_stream(stream, names = None, fmt = '%.18e', delimit = ',', line_terminator = '\n'):    
     """
     Writes elements to an output stream; computes the number of lines written (excluding names). If an element
         is a tuple, its sub-elements will be separated by the delimit byte. Elements will be separated by   
@@ -32,7 +32,7 @@ def chunks_to_stream(stream, names = None, fmt = '%.18e', delimit = b',', line_t
     Examples:
 
     >>> # Reads from a CSV file, and writes the cumulative average to a different one.
-    >>> np.chunk_stream_vals('meteo.csv', 'wind') | np.cum_ave() | np.chunks_to_stream('wind_ave.csv', 'wind')
+    >>> np.chunk_stream_vals('meteo.csv', b'wind') | np.cum_ave() | np.chunks_to_stream('wind_ave.csv', b'wind')
     """
 
     @sinks
