@@ -99,10 +99,10 @@ def chunk_stream_vals(stream,
     Examples:
 
     >>> # Find the correlation between two named columns.
-    >>> np.chunk_stream_vals('meteo.csv', ('day', 'wind'), (float, float), (0, 0)) | np.corr()    
+    >>> np.chunk_stream_vals('meteo.csv', (b'day', b'wind'), (float, float), (0, 0)) | np.corr()    
     0.74
     >>> #Equivalent to:    
-    >>> np.chunk_stream_vals('meteo.csv', ('day', 'wind')) | np.corr()    
+    >>> np.chunk_stream_vals('meteo.csv', (b'day', b'wind')) | np.corr()    
     0.74
 
     >>> # Find the correlation between two indexed columns.
