@@ -13,11 +13,11 @@ using namespace std;
 PyObject * 
 parsed_to_string(const parsed_t & t)
 {
-#if IS_PY3K
+#if PY_MAJOR_VERSION >= 3
     PyObject * const p = PyBytes_FromStringAndSize(t.first, distance(t.first, t.second));
 #else
     PyObject * const p = PyString_FromStringAndSize(t.first, distance(t.first, t.second));
-#endif // #if IS_PY3K
+#endif // #if PY_MAJOR_VERSION >= 3
 
     if (p == NULL)
         PyErr_NoMemory();
@@ -41,7 +41,9 @@ PyObject *
 parsed_to_int(const parsed_t & t)
 {
     bool err;
-    PyObject * const p = PyInt_FromLong(parsed_to_long(t, err));
+    PyObject * const p = pyint_from_long(parsed_to_long(t, err));
+    if (err)
+        return NULL;
     if (p == NULL)
         PyErr_NoMemory();
     return p;
@@ -73,13 +75,13 @@ parsed_to_float(const parsed_t & t)
 char * 
 pystring_as_string(PyObject * o, long & len)
 {
-#if IS_PY3K
+#if PY_MAJOR_VERSION >= 3
     len = PyBytes_Size(o);
     return PyBytes_AsString(o);
-#else // #if IS_PY3K
+#else // #if PY_MAJOR_VERSION >= 3
     len = PyString_Size(o);
     return PyString_AsString(o);
-#endif // #if IS_PY3K
+#endif // #if PY_MAJOR_VERSION >= 3
 }
 
 long *
@@ -101,11 +103,11 @@ parse_longs(PyObject * iterator, long & num, bool & err)
             return NULL;
         }
 
-#if IS_PY3K
+#if PY_MAJOR_VERSION >= 3
         ls[num++] = PyLong_AsLong(obj);
-#else // #if IS_PY3K
+#else // #if PY_MAJOR_VERSION >= 3
         ls[num++] = PyInt_AsLong(obj);
-#endif // #if IS_PY3K
+#endif // #if PY_MAJOR_VERSION >= 3
     }
 
     err = false;
@@ -178,6 +180,6 @@ parse_strings(PyObject * iterator, long & num, bool & err)
 extern "C" PyObject *
 parser_max_field_len(PyObject *module, PyObject *args, PyObject *keyword_args)
 {
-    return PyInt_FromLong(max_field_len);
+    return pyint_from_long(max_field_len);
 }
 
