@@ -6,6 +6,18 @@
 
 #include "defs.hpp"
 
+#if defined(__CYGWIN__) || defined(__CYGWIN32__)
+#define DAGPYPE_NO_USE_AIO
+#endif // #if defined(__CYGWIN__) || defined(__CYGWIN32__)
+
+#if defined(__MINGW32__) || defined(__MINGW64__)
+#define DAGPYPE_NO_USE_AIO
+#endif // #if defined(__MINGW32__) || defined(__MINGW64__)
+
+#if defined(_WIN32) || defined (_WIN64)
+#define DAGPYPE_NO_USE_AIO
+#endif // #if defined(_WIN32) || defined (_WIN64)
+
 #if defined(__GNUC__) && !defined(DAGPYPE_NO_USE_AIO)
 #define DAGPYPE_USE_AIO
 #endif // #if defined(__GNUC__) && !defined(DAGPYPE_NO_USE_AIO)
